package com.jenny.advancedarrows.items;

import com.jenny.advancedarrows.entities.baseArrow;
import com.jenny.advancedarrows.entities.entities;
import net.minecraft.core.Direction;
import net.minecraft.core.Position;
import net.minecraft.core.dispenser.BlockSource;
import net.minecraft.core.dispenser.DefaultDispenseItemBehavior;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.DispenserBlock;
import net.minecraft.world.phys.Vec3;
import net.minecraftforge.eventbus.api.IEventBus;
import net.minecraftforge.registries.DeferredRegister;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.RegistryObject;
import org.jetbrains.annotations.NotNull;

import static com.jenny.advancedarrows.advancedArrows.MODID;

public class items {
    public static final DeferredRegister<Item> ITEMS = DeferredRegister.create(ForgeRegistries.ITEMS, MODID);

    public static final RegistryObject<Item> ARROW_INCENDIARY = ITEMS.register("arrow_incendiary", () -> new ArrowIncendiary(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_RICOCHET = ITEMS.register("arrow_ricochet", () -> new ArrowRicochet(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_KINETIC = ITEMS.register("arrow_kinetic", () -> new ArrowKinetic(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_SHARPENED = ITEMS.register("arrow_sharpened", () -> new ArrowSharpened(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_SWITCH = ITEMS.register("arrow_switch", () -> new ArrowSwitch(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_BREACHING = ITEMS.register("arrow_breaching", () -> new ArrowBreaching(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_HOMING = ITEMS.register("arrow_homing", () -> new ArrowHoming(new Item.Properties()));
    public static final RegistryObject<Item> ARROW_ENDER = ITEMS.register("arrow_ender", () -> new ArrowEnder(new Item.Properties()));

    public static void register(IEventBus bus) {
        ITEMS.register(bus);
    }


    public static void registerDispenser() {
        class ArrowDispenseBehaviour extends DefaultDispenseItemBehavior {
            private final ArrowAbstract.DispenseConfig dispenseConfig;
            private final EntityType<? extends baseArrow> arrowType;

            public ArrowDispenseBehaviour(@NotNull ArrowAbstract arrowItem, EntityType<? extends baseArrow> arrow) {
                this.dispenseConfig = arrowItem.createDispenseConfig();
                this.arrowType = arrow;
            }

            @Override
            public @NotNull ItemStack execute(@NotNull BlockSource pBlockSource, @NotNull ItemStack pItem) {
                Level level = pBlockSource.level();
                Direction dir = pBlockSource.state().getValue(DispenserBlock.FACING);
                Position pos = this.dispenseConfig.positionFunction().getDispensePosition(pBlockSource, dir);

                AbstractArrow arrowEntity = arrowType.create(level);
                assert arrowEntity != null;
                arrowEntity.setPos(new Vec3(pos.x(), pos.y(), pos.z()));
                arrowEntity.setDeltaMovement(dir.getStepX(), dir.getStepY(), dir.getStepZ());
                level.addFreshEntity(arrowEntity);
                pItem.shrink(1);
                return pItem;
            }
        }

        DispenserBlock.registerBehavior(ARROW_INCENDIARY.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_INCENDIARY.get(), entities.ARROW_INCENDIARY.get()));
        DispenserBlock.registerBehavior(ARROW_RICOCHET.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_RICOCHET.get(), entities.ARROW_RICOCHET.get()));
        DispenserBlock.registerBehavior(ARROW_KINETIC.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_KINETIC.get(), entities.ARROW_KINETIC.get()));
        DispenserBlock.registerBehavior(ARROW_SHARPENED.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_SHARPENED.get(), entities.ARROW_SHARPENED.get()));
        DispenserBlock.registerBehavior(ARROW_SWITCH.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_SWITCH.get(), entities.ARROW_SWITCH.get()));
        DispenserBlock.registerBehavior(ARROW_BREACHING.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_BREACHING.get(), entities.ARROW_BREACHING.get()));
        DispenserBlock.registerBehavior(ARROW_HOMING.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_HOMING.get(), entities.ARROW_HOMING.get()));
        DispenserBlock.registerBehavior(ARROW_ENDER.get(), new ArrowDispenseBehaviour(
                (ArrowAbstract) items.ARROW_ENDER.get(), entities.ARROW_ENDER.get()));
    }
}
